@@ -32,7 +32,7 @@ func init() {
 
 // I2C on the Arduino Nano 33.
 var (
-	I2C0 = I2C{
+	I2C0 = &I2C{
 		Bus:    sam.SERCOM4_I2CM,
 		SERCOM: 4,
 	}
