@@ -16,7 +16,7 @@ const (
 )
 
 type Value struct {
-	typecode Type
+	typecode rawType
 	value    unsafe.Pointer
 	flags    valueFlags
 }
@@ -28,6 +28,13 @@ func (v Value) isIndirect() bool {
 	return v.flags&valueFlagIndirect != 0
 }
 
+// isExported returns whether the value represented by this Value could be
+// accessed without violating type system constraints. For example, it is not
+// set for unexported struct fields.
+func (v Value) isExported() bool {
+	return v.flags&valueFlagExported != 0
+}
+
 func Indirect(v Value) Value {
 	if v.Kind() != Ptr {
 		return v
@@ -36,10 +43,10 @@ func Indirect(v Value) Value {
 }
 
 //go:linkname composeInterface runtime.composeInterface
-func composeInterface(Type, unsafe.Pointer) interface{}
+func composeInterface(rawType, unsafe.Pointer) interface{}
 
 //go:linkname decomposeInterface runtime.decomposeInterface
-func decomposeInterface(i interface{}) (Type, unsafe.Pointer)
+func decomposeInterface(i interface{}) (rawType, unsafe.Pointer)
 
 func ValueOf(i interface{}) Value {
 	typecode, value := decomposeInterface(i)
@@ -51,11 +58,20 @@ func ValueOf(i interface{}) Value {
 }
 
 func (v Value) Interface() interface{} {
-	if v.isIndirect() && v.Type().Size() <= unsafe.Sizeof(uintptr(0)) {
+	if !v.isExported() {
+		panic("(reflect.Value).Interface: unexported")
+	}
+	return valueInterfaceUnsafe(v)
+}
+
+// valueInterfaceUnsafe is used by the runtime to hash map keys. It should not
+// be subject to the isExported check.
+func valueInterfaceUnsafe(v Value) interface{} {
+	if v.isIndirect() && v.typecode.Size() <= unsafe.Sizeof(uintptr(0)) {
 		// Value was indirect but must be put back directly in the interface
 		// value.
 		var value uintptr
-		for j := v.Type().Size(); j != 0; j-- {
+		for j := v.typecode.Size(); j != 0; j-- {
 			value = (value << 8) | uintptr(*(*uint8)(unsafe.Pointer(uintptr(v.value) + j - 1)))
 		}
 		v.value = unsafe.Pointer(value)
@@ -67,8 +83,16 @@ func (v Value) Type() Type {
 	return v.typecode
 }
 
+// Internal function only, do not use.
+//
+// RawType returns the raw, underlying type code. It is used in the runtime
+// package and needs to be exported for the runtime package to access it.
+func (v Value) RawType() rawType {
+	return v.typecode
+}
+
 func (v Value) Kind() Kind {
-	return v.Type().Kind()
+	return v.typecode.Kind()
 }
 
 // IsNil returns whether the value is the nil value. It panics if the value Kind
@@ -132,7 +156,7 @@ func (v Value) CanInterface() bool {
 }
 
 func (v Value) CanAddr() bool {
-	panic("unimplemented: (reflect.Value).CanAddr()")
+	return v.flags&(valueFlagIndirect) == valueFlagIndirect
 }
 
 func (v Value) Addr() Value {
@@ -313,10 +337,9 @@ func chanlen(p unsafe.Pointer) int
 // Len returns the length of this value for slices, strings, arrays, channels,
 // and maps. For other types, it panics.
 func (v Value) Len() int {
-	t := v.Type()
-	switch t.Kind() {
+	switch v.typecode.Kind() {
 	case Array:
-		return v.Type().Len()
+		return v.typecode.Len()
 	case Chan:
 		return chanlen(v.value)
 	case Map:
@@ -336,10 +359,9 @@ func chancap(p unsafe.Pointer) int
 // Cap returns the capacity of this value for arrays, channels and slices.
 // For other types, it panics.
 func (v Value) Cap() int {
-	t := v.Type()
-	switch t.Kind() {
+	switch v.typecode.Kind() {
 	case Array:
-		return v.Type().Len()
+		return v.typecode.Len()
 	case Chan:
 		return chancap(v.value)
 	case Slice:
@@ -352,7 +374,7 @@ func (v Value) Cap() int {
 // NumField returns the number of fields of this struct. It panics for other
 // value types.
 func (v Value) NumField() int {
-	return v.Type().NumField()
+	return v.typecode.NumField()
 }
 
 func (v Value) Elem() Value {
@@ -366,7 +388,7 @@ func (v Value) Elem() Value {
 			return Value{}
 		}
 		return Value{
-			typecode: v.Type().Elem(),
+			typecode: v.typecode.elem(),
 			value:    ptr,
 			flags:    v.flags | valueFlagIndirect,
 		}
@@ -377,7 +399,7 @@ func (v Value) Elem() Value {
 
 // Field returns the value of the i'th field of this struct.
 func (v Value) Field(i int) Value {
-	structField := v.Type().Field(i)
+	structField := v.typecode.rawField(i)
 	flags := v.flags
 	if structField.PkgPath != "" {
 		// The fact that PkgPath is present means that this field is not
@@ -385,14 +407,15 @@ func (v Value) Field(i int) Value {
 		flags &^= valueFlagExported
 	}
 
-	size := v.Type().Size()
-	fieldSize := structField.Type.Size()
+	size := v.typecode.Size()
+	fieldType := structField.Type
+	fieldSize := fieldType.Size()
 	if v.isIndirect() || fieldSize > unsafe.Sizeof(uintptr(0)) {
 		// v.value was already a pointer to the value and it should stay that
 		// way.
 		return Value{
 			flags:    flags,
-			typecode: structField.Type,
+			typecode: fieldType,
 			value:    unsafe.Pointer(uintptr(v.value) + structField.Offset),
 		}
 	}
@@ -407,7 +430,7 @@ func (v Value) Field(i int) Value {
 		// situation explicitly.
 		return Value{
 			flags:    flags,
-			typecode: structField.Type,
+			typecode: fieldType,
 			value:    unsafe.Pointer(uintptr(0)),
 		}
 	}
@@ -420,7 +443,7 @@ func (v Value) Field(i int) Value {
 		value := unsafe.Pointer(loadValue(ptr, fieldSize))
 		return Value{
 			flags:    0,
-			typecode: structField.Type,
+			typecode: fieldType,
 			value:    value,
 		}
 	}
@@ -430,7 +453,7 @@ func (v Value) Field(i int) Value {
 	value := maskAndShift(uintptr(v.value), structField.Offset, fieldSize)
 	return Value{
 		flags:    flags,
-		typecode: structField.Type,
+		typecode: fieldType,
 		value:    unsafe.Pointer(value),
 	}
 }
@@ -444,10 +467,10 @@ func (v Value) Index(i int) Value {
 			panic("reflect: slice index out of range")
 		}
 		elem := Value{
-			typecode: v.Type().Elem(),
+			typecode: v.typecode.elem(),
 			flags:    v.flags | valueFlagIndirect,
 		}
-		addr := uintptr(slice.Data) + elem.Type().Size()*uintptr(i) // pointer to new value
+		addr := uintptr(slice.Data) + elem.typecode.Size()*uintptr(i) // pointer to new value
 		elem.value = unsafe.Pointer(addr)
 		return elem
 	case String:
@@ -464,13 +487,13 @@ func (v Value) Index(i int) Value {
 		}
 	case Array:
 		// Extract an element from the array.
-		elemType := v.Type().Elem()
+		elemType := v.typecode.elem()
 		elemSize := elemType.Size()
-		size := v.Type().Size()
+		size := v.typecode.Size()
 		if size == 0 {
 			// The element size is 0 and/or the length of the array is 0.
 			return Value{
-				typecode: v.Type().Elem(),
+				typecode: v.typecode.elem(),
 				flags:    v.flags,
 			}
 		}
@@ -481,7 +504,7 @@ func (v Value) Index(i int) Value {
 			// elemSize.
 			addr := uintptr(v.value) + elemSize*uintptr(i) // pointer to new value
 			return Value{
-				typecode: v.Type().Elem(),
+				typecode: v.typecode.elem(),
 				flags:    v.flags,
 				value:    unsafe.Pointer(addr),
 			}
@@ -492,7 +515,7 @@ func (v Value) Index(i int) Value {
 			// Load the value from the pointer.
 			addr := uintptr(v.value) + elemSize*uintptr(i) // pointer to new value
 			return Value{
-				typecode: v.Type().Elem(),
+				typecode: v.typecode.elem(),
 				flags:    v.flags,
 				value:    unsafe.Pointer(loadValue(unsafe.Pointer(addr), elemSize)),
 			}
@@ -503,7 +526,7 @@ func (v Value) Index(i int) Value {
 		offset := elemSize * uintptr(i)
 		value := maskAndShift(uintptr(v.value), offset, elemSize)
 		return Value{
-			typecode: v.Type().Elem(),
+			typecode: v.typecode.elem(),
 			flags:    v.flags,
 			value:    unsafe.Pointer(value),
 		}
@@ -532,6 +555,14 @@ func maskAndShift(value, offset, size uintptr) uintptr {
 	return (uintptr(value) >> (offset * 8)) & mask
 }
 
+func (v Value) NumMethod() int {
+	return v.typecode.NumMethod()
+}
+
+func (v Value) OverflowFloat(x float64) bool {
+	panic("unimplemented: (reflect.Value).OverflowFloat()")
+}
+
 func (v Value) MapKeys() []Value {
 	panic("unimplemented: (reflect.Value).MapKeys()")
 }
@@ -561,10 +592,10 @@ func (it *MapIter) Next() bool {
 
 func (v Value) Set(x Value) {
 	v.checkAddressable()
-	if !v.Type().AssignableTo(x.Type()) {
+	if !v.typecode.AssignableTo(x.typecode) {
 		panic("reflect: cannot set")
 	}
-	size := v.Type().Size()
+	size := v.typecode.Size()
 	xptr := x.value
 	if size <= unsafe.Sizeof(uintptr(0)) && !x.isIndirect() {
 		value := x.value
@@ -655,6 +686,18 @@ func (v Value) SetString(x string) {
 	}
 }
 
+func (v Value) SetBytes(x []byte) {
+	panic("unimplemented: (reflect.Value).SetBytes()")
+}
+
+func (v Value) SetCap(n int) {
+	panic("unimplemented: (reflect.Value).SetCap()")
+}
+
+func (v Value) SetLen(n int) {
+	panic("unimplemented: (reflect.Value).SetLen()")
+}
+
 func (v Value) checkAddressable() {
 	if !v.isIndirect() {
 		panic("reflect: value is not addressable")
@@ -681,8 +724,14 @@ func Zero(typ Type) Value {
 	panic("unimplemented: reflect.Zero()")
 }
 
+// New is the reflect equivalent of the new(T) keyword, returning a pointer to a
+// new value of the given type.
 func New(typ Type) Value {
-	panic("unimplemented: reflect.New()")
+	return Value{
+		typecode: PtrTo(typ).(rawType),
+		value:    alloc(typ.Size()),
+		flags:    valueFlagExported,
+	}
 }
 
 type funcHeader struct {
@@ -713,6 +762,9 @@ func (e *ValueError) Error() string {
 // llvm.memcpy.p0i8.p0i8.i32().
 func memcpy(dst, src unsafe.Pointer, size uintptr)
 
+//go:linkname alloc runtime.alloc
+func alloc(size uintptr) unsafe.Pointer
+
 // Copy copies the contents of src into dst until either
 // dst has been filled or src has been exhausted.
 func Copy(dst, src Value) int {
